/* Copyright (C) 1998 Odysseas Pentakalos and Aram Khalili

  This program is freely distributable under the GNU General
  Public License Version 2 or later. The GNU GPL is available
  from the Free Software Foundation, Inc., 59 Temple Place,
  Suite 330, Boston, MA 02111-1307, USA.

  The authors can be contacted by email :
  Odysseas Pentakalos : odysseas@cs.umbc.edu
  Aram Khalili        : akhali1@gl.umbc.edu
*/

#ifndef _TAPELIB_H_
#define _TAPELIB_H_

#include <sys/types.h>

/*
  SCSI Opcodes
    For some reason /usr/include/linux/scsi/scsi.h defines the
    POSITION_TO_ELEMENT command as a SEEK_10
    */
#define POSITION_TO_ELEMENT SEEK_10

/*
  These are the type codes of elements that the tape library can
  report on
  */
#define ALL_ELT_TYPES 0
#define MED_TRANSP_ELT_TYPE 1
#define STOR_ELT_TYPE 2
#define DATA_TRANSF_ELT_TYPE 4

#define ELT_STAT_DATA 8
#define ELT_STAT_PAGE 8
#define ELT_GROUPS 1

#define MED_TRANSP_ELT 16
#define STOR_ELT 16
#define DATA_TRANSF_ELT 16
#define VOL_TAG 36
#define TAPE_DRIVE_SN 10

#define TWOBYTEMSB(twobyteword) ((twobyteword & 0xff00) >> 8)
#define TWOBYTELSB(twobyteword) (twobyteword & 0x00ff)

#define THREEBYTEMSB(threebyteword) ((threebyteword & 0xff0000) >> 16)
#define THREEBYTEMID(threebyteword) ((threebyteword & 0x00ff00) >> 8)
#define THREEBYTELSB(threebyteword) (threebyteword & 0x0000ff)

typedef struct {
  u_int voltag:1;
  u_int snreq:1;
  u_int type:3;
  __u16 start;
  u_int initflag;
  u_int num_med_transp_elt;
  u_int num_stor_elt;
  u_int num_data_transf_elt;
  u_int alloc_len;
} elt_stat_req_t;

typedef struct {
  u_int first_elt;
  u_int num_elt;
  u_int elt_stat_page_bytes;
} elt_stat_data_t;

typedef struct {
  u_int elt_type;
  u_char pvoltag_flag;
  u_int elt_desc_len;
  u_int elt_desc_len_tot;
  u_int elt_desc_cnt;
} elt_stat_page_t;

typedef struct {
  u_int elt_addr;
  u_char access;
  u_char except;
  u_char full;
  u_char add_sense;
  u_char add_sense_qual;
  u_char svalid;
  u_char invert;
  u_int  source_elt_addr;
} stor_elt_desc_t;

typedef struct {
  u_int elt_addr;
  u_char access;
  u_char except;
  u_char full;
  u_char add_sense;
  u_char add_sense_qual;
  u_char idvalid;
  u_char luvalid;
  u_char lun;
  u_char scsi_addr;
  u_char svalid;
  u_char invert;
  u_int  source_elt_addr;
} data_transf_elt_desc_t;

typedef struct {
  u_int elt_addr;
  u_char except;
  u_char full;
  u_char add_sense;
  u_char add_sense_qual;
  u_char svalid;
  u_char invert;
  u_int  source_elt_addr;
} med_transp_elt_desc_t;

typedef struct {
  u_char ps;
  __u16 tea;
  __u16 num_trans_elt;
  __u16 sea;
  __u16 num_stor_elt;
  __u16 iea;
  __u16 num_imp_elt;
  __u16 dea;
  __u16 num_data_elt;
} elt_addr_assgn_t;

typedef struct {
  u_char ps;
  u_char rot;
  u_char trans_elt_no;
} tgd_t;

typedef struct {
  u_char ps;
  u_char dt;
  u_char ie;
  u_char st;
  u_char mt;
} dev_cap_t;

typedef struct {
  u_char ps;
  u_char LCD_sec;
  u_char wl1;
  u_char wl2;
  u_char wl3;
  u_char wl4;
  char line1[20];
  char line2[20];
  char line3[20];
  char line4[20];
} LCD_t;

typedef struct {
  u_char ps;
  u_char par;
  u_char retries;
} parity_t;

int move_medium(int fd, __u16 tea, __u16 src, __u16 dest);

int position_elt(int fd, __u16 tea, __u16 dest);

int read_elt_stat(int fd, elt_stat_req_t *esr, elt_stat_data_t *esd,
		  elt_stat_page_t **esp, stor_elt_desc_t **sed,
		  data_transf_elt_desc_t **dted,
		  med_transp_elt_desc_t **mted);

int get_elt_stat_data(u_char *buf, elt_stat_data_t *esd);

void print_elt_stat_data(elt_stat_data_t *esd);

int get_elt_stat_page(u_char *buf, elt_stat_page_t *esp);

void print_elt_stat_page(elt_stat_page_t *esp);

int get_stor_elt_desc(u_char *buf, stor_elt_desc_t *sed);

void print_stor_elt_desc(stor_elt_desc_t *sed);

int get_med_transp_elt_desc(u_char *buf, med_transp_elt_desc_t *mted);

int get_data_transf_elt_desc(u_char *buf, data_transf_elt_desc_t *dted);

int load(int fd, int luflag);

#endif _TAPELIB_H_










